@@ -49,7 +49,7 @@ module Puppet::ModuleTool
           File.open(metadata_path) do |f|
             begin
               @metadata.update(Puppet::Util::Json.load(f))
-            rescue Puppet::Util::Json::ParserError => ex
+            rescue Puppet::Util::Json::ParseError => ex
               raise ArgumentError, _("Could not parse JSON %{metadata_path}") % { metadata_path: metadata_path }, ex.backtrace
             end
           end
