@@ -25,19 +25,25 @@ class Puppet::Network::HTTP::Factory
   def create_connection(site)
     Puppet.debug("Creating new connection for #{site}")
 
-    http = Puppet::Util::HttpProxy.proxy(URI(site.addr))
+    args = [site.host, site.port]
+
+    unless Puppet::Util::HttpProxy.no_proxy?(site)
+      if Puppet[:http_proxy_host] == "none"
+        args << nil << nil
+      else
+        args << Puppet[:http_proxy_host] << Puppet[:http_proxy_port]
+      end
+    end
+
+    http = Net::HTTP.new(*args)
     http.use_ssl = site.use_ssl?
     http.read_timeout = Puppet[:http_read_timeout]
     http.open_timeout = Puppet[:http_connect_timeout]
     http.keep_alive_timeout = KEEP_ALIVE_TIMEOUT if http.respond_to?(:keep_alive_timeout=)
 
     if Puppet[:sourceaddress]
-      if http.respond_to?(:local_host)
-        Puppet.debug("Using source IP #{Puppet[:sourceaddress]}")
-        http.local_host = Puppet[:sourceaddress]
-      else
-        raise ArgumentError, "Setting 'sourceaddress' is unsupported by this version of Net::HTTP."
-      end
+      Puppet.debug("Using source IP #{Puppet[:sourceaddress]}")
+      http.local_host = Puppet[:sourceaddress]
     end
 
     if Puppet[:http_debug]
