@@ -8,7 +8,6 @@ class Puppet::Parser::Resource < Puppet::Resource
   require 'puppet/util/tagging'
 
   include Puppet::Util
-  include Puppet::Util::MethodHelper
   include Puppet::Util::Errors
   include Puppet::Util::Logging
 
@@ -169,13 +168,20 @@ class Puppet::Parser::Resource < Puppet::Resource
     end
 
     if evaluated?
+      error_location_str = Puppet::Util::Errors.error_location(file, line)
+      msg = if error_location_str.empty?
+              _('Attempt to override an already evaluated resource with new values')
+            else
+              _('Attempt to override an already evaluated resource, defined at %{error_location}, with new values') % { error_location: error_location_str }
+            end
       strict = Puppet[:strict]
       unless strict == :off
-        msg = _('Attempt to override an already evaluated resource%{location} with new values') %
-          { resource: ref, location: append_location(file, line, _(', defined'), ',') }
-        raise Puppet::ParseError.new(msg, resource.file, resource.line) if strict == :error
-        msg << append_location(resource.file, resource.line)
-        Puppet.warning(msg)
+        if strict == :error
+          raise Puppet::ParseError.new(msg, resource.file, resource.line)
+        else
+          msg += Puppet::Util::Errors.error_location_with_space(resource.file, resource.line)
+          Puppet.warning(msg)
+        end
       end
     end
 
@@ -322,32 +328,10 @@ class Puppet::Parser::Resource < Puppet::Resource
 
   private
 
-  # Append location to the message if location is present. If it is not, an empty string
-  # is returned instead.
-  #
-  # @param file [String] path to the file
-  # @param line [Integer] the line in the file
-  # @param prefix [String] string prefix
-  # @param suffix [String] string suffix
-  # @return [String] the formatted location or an empty string
-  def append_location(file, line, prefix = '', suffix = '')
-    if file && file != ''
-      if line
-        _('%{prefix} at %{file}:%{line}%{suffix}') % { prefix: prefix, suffix: suffix, file: file, line: line }
-      else
-        _('%{prefix} in %{file}%{suffix}') % { prefix: prefix, suffix: suffix, file: file }
-      end
-    elsif line
-      _('%{prefix} at line %{line}%{suffix}') % { prefix: prefix, suffix: suffix, line: line }
-    else
-      ''
-    end
-  end
-
   def add_scope_tags
     scope_resource = scope.resource
     unless scope_resource.nil? || scope_resource.equal?(self)
-      merge_tags(scope_resource)
+      merge_tags_from(scope_resource)
     end
   end
 
@@ -371,10 +355,24 @@ class Puppet::Parser::Resource < Puppet::Resource
 
     # The parameter is already set.  Fail if they're not allowed to override it.
     unless param.source.child_of?(current.source) || param.source.equal?(current.source) && scope.is_default?(type, param.name, current.value)
-      msg = _("Parameter '%{name}' is already set on %{resource}") % { name: param.name, resource: ref }
-      msg << _(' by %{source}') % { source: current.source.to_s } unless current.source.to_s == ''
-      msg << append_location(current.file, current.line)
-      msg << _('; cannot redefine')
+      error_location_str = Puppet::Util::Errors.error_location(current.file, current.line)
+      msg = if current.source.to_s == ''
+              if error_location_str.empty?
+                _("Parameter '%{name}' is already set on %{resource}; cannot redefine") %
+                    { name: param.name, resource: ref }
+              else
+                _("Parameter '%{name}' is already set on %{resource} at %{error_location}; cannot redefine") %
+                    { name: param.name, resource: ref, error_location: error_location_str }
+              end
+            else
+              if error_location_str.empty?
+                _("Parameter '%{name}' is already set on %{resource} by %{source}; cannot redefine") %
+                    { name: param.name, resource: ref, source: current.source.to_s }
+              else
+                _("Parameter '%{name}' is already set on %{resource} by %{source} at %{error_location}; cannot redefine") %
+                    { name: param.name, resource: ref, source: current.source.to_s, error_location: error_location_str }
+              end
+            end
       raise Puppet::ParseError.new(msg, param.file, param.line)
     end
 
