@@ -12,12 +12,12 @@ module Puppet::Util::InstanceLoader
   end
 
   # Define a new type of autoloading.
-  def instance_load(type, path, options = {})
+  def instance_load(type, path)
     @autoloaders ||= {}
     @instances ||= {}
     type = type.intern
     @instances[type] = {}
-    @autoloaders[type] = Puppet::Util::Autoload.new(self, path, options)
+    @autoloaders[type] = Puppet::Util::Autoload.new(self, path)
 
     # Now define our new simple methods
     unless respond_to?(type)
@@ -32,24 +32,6 @@ module Puppet::Util::InstanceLoader
     @instances[type].keys
   end
 
-  # Collect the docs for all of our instances.
-  def instance_docs(type)
-    docs = ""
-
-    # Load all instances.
-    instance_loader(type).loadall
-
-    # Use this method so they all get loaded
-    loaded_instances(type).sort_by(&:to_s).each do |name|
-      mod = self.loaded_instance(type, name)
-      docs << "#{name}\n#{"-" * name.to_s.length}\n"
-
-      docs << Puppet::Util::Docs.scrub(mod.doc) << "\n\n"
-    end
-
-    docs
-  end
-
   # Return the instance hash for our type.
   def instance_hash(type)
     @instances[type.intern]
@@ -65,7 +47,7 @@ module Puppet::Util::InstanceLoader
     name = name.intern
     return nil unless instances = instance_hash(type)
     unless instances.include? name
-      if instance_loader(type).load(name)
+      if instance_loader(type).load(name, Puppet.lookup(:current_environment))
         unless instances.include? name
           Puppet.warning(_("Loaded %{type} file for %{name} but %{type} was not defined") % { type: type, name: name })
           return nil
